/**************************************************************************
*
* Function:  initPLL_2126x -- Initialize the PLL of ADSP-21261/2/6
*
*
*            #include <initPLL.h> 
* Prototype: void initPLL_2126x(int multi,int div,int sdckr,boolean indiv);       
*
* initPLL_21262 function usage
*
* S.No  parameter name	Input range/values	  Description	 
*
* 1.    multi			0 - 63        		  Chooses the multiplier value from 0 to 63. Value of 0 corresponds to 64.
* 2.    div             0 ,1,2 and 3 		  Chooses the divider value,   0 - Divider value 2
*																		   1 - Divider value 4
*																		   2 - Divider value 8
*																		   3 - Divider value 16
* 3.    sdckr           dummy value 	  	  Not used.					  
* 4.	indiv			true , false          True - Enables the Input Divider		      																		  																	   
*											  False - Disables the Input Divider	
*            if ((multi > 0) && (div > 0))
*			 {
*				if( indiv == FALSE )
*				{
*					CCLK = CLKIN * multi / div;
*				}
*				else
*				{
*					CCLK = 2 * CLKIN * multi / div;
*				}
*
*				
*			}
*			else if( multi < 0 )
*			{
*				if( indiv == FALSE )
*				{
*					CCLK = CLKIN * multi;
*				}
*				else
*				{
*					CCLK = CLKIN * multi / 2
*				}
*				
*			}
*			else 
*			{
*				if( indiv == FALSE )
*				{
*					CCLK = CLKIN / div;
*				}
*				else
*				{
*					CCLK = CLKIN / div * 2
*				}
*				
*			}
*					
* 				
*
* (c) Copyright 2002-2004 Analog Devices, Inc.  All rights reserved.
*     $Revision: 1.5 $
**************************************************************************/

.file_attr libGroup="initPLL.h";
.file_attr libName="libc";
.file_attr libFunc="_initPLL_2126x";
.file_attr FuncName="_initPLL_2126x";
.file_attr prefersMem="internal";
.file_attr prefersMemNum="30";

#include "lib_glob.h"
#include <def21266.h>


.SEGMENT/CODE Code_Space_Name; // Sections used by the Visual DSP++ tool for the
			       // library function
.FILE RTL_FILENAME;

.GLOBAL   _initPLL_2126x;

_initPLL_2126x:
	
    r1 = dm(1,i6); 
    
   	//ustat1 = 0;   
   	ustat1 = dm(PMCTL);
    bit clr ustat1 0x83FF;
   	bit set ustat1 DIVEN; // Enable the divisor for the default value 
   	    
    r0 = 0;
    
check_for_indiv:   
	comp(r1,r0);	// Check whether the input divider is enabled.
    if le jump check_for_div;
	bit set ustat1 INDIV;   // If input divider is enabled set INDIV bit.

check_for_div:       
    comp(r8,r0); // Check whether the divider is used.
    if le jump check_for_mult; 
    r2 = 0x3;	
    r8 = r8 and r2; // Extract the divisor bits
    r8 = lshift r8 by 6;	
    r2 = ustat1;
    r2 = r2 or r8;
    ustat1 = r2;	// set the divisor value in the ustat1
           
check_for_mult:
    comp(r4,r0); // Check whether the multiplier is used.
    if ge jump change_pmctl;
    dm(PMCTL) = ustat1;	// initialize the PMCTL register
    jump divider_loop;
    
change_pmctl:  
    r2 = 0x3F;
    r4 = r4 and r2; // Extract the multiplier bits
    r2 = ustat1;
    r2 = r2 or r4;
    ustat1 = r2;	// set the multiplier value in the ustat1
	nop;  	
       
    dm(PMCTL) = ustat1; // initialize the PMCTL register
    
    bit set ustat1 PLLBP; //Place the PLL in bypass mode.
	bit clr ustat1 DIVEN; //Clear the DIVEN bit while placing the PLL in bypass mode.
    dm(PMCTL) = ustat1;

     // Wait for at least 4096 cycles for the pll to lock
    lcntr = 5000, do loopend2 until lce;
loopend2:     nop;

    ustat1 = dm(PMCTL);	//Reading the PMCTL register will return the DIVEN bit value as zero.
    bit clr ustat1 PLLBP; //Take the PLL out of bypass mode.
    dm(PMCTL) = ustat1;
    
divider_loop:
    // wait for atleast 15 CCLK cycles for the new divider values to take effect
    lcntr = 16, do loopend3 until lce;
loopend3:     nop;

exit_initPLL:           

	
    FETCH_RETURN // Instruction used by the Visual DSP++ tools library for
		 // returning from the library function
    RETURN (DB);
    RESTORE_STACK
    RESTORE_FRAME
    
_initPLL_2126x.end:
.ENDSEG;